@@ -1,4 +1,4 @@
-/* splitflap, Copyright (c) 2015 Jamie Zawinski <jwz@jwz.org>
+/* splitflap, Copyright (c) 2015-2018 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -26,7 +26,7 @@
 			"*program:      xscreensaver-text\n" \
 			"*usePty:       False\n"
 
-# define refresh_splitflap 0
+# define release_splitflap 0
 #undef countof
 #define countof(x) (sizeof((x))/sizeof((*x)))
 
@@ -46,7 +46,7 @@
 
 #include "gltrackball.h"
 #include "rotator.h"
-#include "xpm-ximage.h"
+#include "ximage-loader.h"
 #include "utf8wc.h"
 #include "textclient.h"
 #include "texfont.h"
@@ -299,14 +299,7 @@ init_splitflap (ModeInfo *mi)
   splitflap_configuration *bp;
   int wire = MI_IS_WIREFRAME(mi);
   int i;
-  if (!bps) {
-    bps = (splitflap_configuration *)
-      calloc (MI_NUM_SCREENS(mi), sizeof (splitflap_configuration));
-    if (!bps) {
-      fprintf(stderr, "%s: out of memory\n", progname);
-      exit(1);
-    }
-  }
+  MI_INIT (mi, bps);
 
   bp = &bps[MI_SCREEN(mi)];
   bp->glx_context = init_GL(mi);
@@ -1401,11 +1394,12 @@ draw_splitflap (ModeInfo *mi)
 }
 
 ENTRYPOINT void
-release_splitflap (ModeInfo *mi)
+free_splitflap (ModeInfo *mi)
 {
   splitflap_configuration *bp = &bps[MI_SCREEN(mi)];
   if (bp->tc)
     textclient_close (bp->tc);
+  bp->tc = 0;
   /* #### bp->texinfo */
 }
 
